using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinSchedule;
using Infragistics.Win;
using Infragistics.Win.UltraWinScrollBar;
using Infragistics.Win.UltraWinToolTip;

namespace UltraTimeLineView_CS
{
    public partial class Form1 : Form
    {
        #region Private Members

        private static Random rnd = new Random(randomSeed);
        private const int randomSeed = 0;
       // private System.Resources.ResourceManager rm = new System.Resources.ResourceManager(typeof(Form1));
        private System.Resources.ResourceManager rm = UltraTimeLineView_CS.Properties.Resources.ResourceManager;

        #endregion Private Members

        #region Constructor

        public Form1()
        {
            InitializeComponent();
        }

        #endregion Constructor

        #region Initialization

        #region InitializeSampleControls
        /// <summary>
        /// Initializes all the property-modifying controls
        /// </summary>
        private void InitializeSampleControls()
        {
            // Populate the Owners dropdown.
            this.cmbOwners.DataSource = WinScheduleDataCreationUtils.Owners;
            this.cmbOwners.SelectedIndex = -1;
            this.cmbOwners.SelectedValueChanged += new EventHandler(cmbOwners_SelectedValueChanged);

            // Set the Primary Interval to 15 minutes.
            this.cmbPrimaryInterval.Items.AddRange(new string[] { 
                                                    rm.GetString("5 Minutes"), 
                                                    rm.GetString("15 Minutes"), 
                                                    rm.GetString("30 Minutes"), 
                                                    rm.GetString("1 Hour"), 
                                                    rm.GetString("2 Hours"), 
                                                    rm.GetString("4 Hours"), 
                                                    rm.GetString("6 Hours"), 
                                                    rm.GetString("1 Day") });
            this.cmbPrimaryInterval.SelectedIndex = 1;
            this.cmbPrimaryInterval.SelectedIndexChanged += new EventHandler(cmbPrimaryInterval_SelectedIndexChanged);

            // Check the items in the Appointment Display listbox that default to True in the AppointmentDisplaySettings.
            this.clbAppointmentDisplayItems.SetItemChecked(0, true); // Subject
            this.clbAppointmentDisplayItems.SetItemChecked(1, true); // Location
            this.clbAppointmentDisplayItems.SetItemChecked(4, true); // DateTime Continuation Indicator Buttons
            this.clbAppointmentDisplayItems.SetItemChecked(5, true); // BarColor Indicator
            this.clbAppointmentDisplayItems.ItemCheck += new ItemCheckEventHandler(clbAppointmentDisplayItems_ItemCheck);

            // Populate the AppointmentDragMode dropdown.
            this.cmbAppointmentDragMode.DataSource = Enum.GetValues(typeof(TimelineViewAppointmentDragMode));
            this.cmbAppointmentDragMode.SelectedItem = TimelineViewAppointmentDragMode.Default;

            // Populate the AppointmentTooltipVisibility dropdown.
            this.cmbAppointmentTooltips.DataSource = Enum.GetValues(typeof(TimelineViewActivityToolTipVisibility));
            this.cmbAppointmentTooltips.SelectedItem = TimelineViewActivityToolTipVisibility.Default;

            // Populate the AppointmentVisibility dropdown.
            this.cmbAppointmentVisibility.DataSource = Enum.GetValues(typeof(TimelineViewAppointmentVisibility));
            this.cmbAppointmentVisibility.SelectedItem = TimelineViewAppointmentVisibility.Default;

            // Populate the ColumnSizing dropdown.
            this.cmbColumnSizing.DataSource = Enum.GetValues(typeof(TimelineViewColumnSizing));
            this.cmbColumnSizing.SelectedItem = TimelineViewColumnSizing.Header;

            // Populate the ColumnAutoSizing dropdown.
            this.cmbColumnAutoSizing.DataSource = Enum.GetValues(typeof(TimelineViewColumnAutoSizing));
            this.cmbColumnAutoSizing.SelectedItem = TimelineViewColumnAutoSizing.PrimaryIntervalOnly;

            // Set the initial value for the ColumnWidth editor.
            this.nmColumnWidth.Value = this.ultraTimelineView1.ColumnWidthResolved;

            // Set the initial value for the OwnersInView editor.
            this.nmOwnersInView.Value = Convert.ToInt32(this.ultraTimelineView1.OwnersInView);

            // Populate the OwnerGroupingStyle dropdown
            this.cmbOwnerGroupingStyle.DataSource = Enum.GetValues(typeof(TimelineViewOwnerGroupingStyle));
            this.cmbOwnerGroupingStyle.SelectedItem = TimelineViewOwnerGroupingStyle.Default;

            // Populate the HolidayVisibility dropdown.
            this.cmbHolidayVisibility.DataSource = Enum.GetValues(typeof(TimelineViewHolidayVisibility));
            this.cmbHolidayVisibility.SelectedItem = TimelineViewHolidayVisibility.ShowForAllOwners;

            // Populate the Holidays listbox with the Holidays from the UltraCalendarInfo
            foreach (Holiday holiday in this.ultraCalendarInfo1.Holidays)
                this.lbHolidays.Items.Add(holiday.StartDate.ToShortDateString());

            // Populate the HolidayTooltipVisibility dropdown.
            this.cmbHolidayTooltipVisibility.DataSource = Enum.GetValues(typeof(TimelineViewActivityToolTipVisibility));
            this.cmbHolidayTooltipVisibility.SelectedItem = TimelineViewActivityToolTipVisibility.Default;

            // Populate the OwnerSizing dropdown.
            this.cmbOwnerSizing.DataSource = Enum.GetValues(typeof(TimelineViewOwnerSizing));
            this.cmbOwnerSizing.SelectedItem = TimelineViewOwnerSizing.HeaderOrRow;

            // Populate the ViewStyle dropdown.
            this.cmbViewStyle.DataSource = Enum.GetValues(typeof(ViewStyle));
            this.cmbViewStyle.SelectedItem = ViewStyle.Office2007;

            // Set the OwnerTextOrientation dropdown to the default "Horizontal 90 degrees"
            this.cmbOwnerTextOrientation.Items.AddRange(new string[] { 
                                                    rm.GetString("Horizontal90"), 
                                                    rm.GetString("Horizontal60"), 
                                                    rm.GetString("Horizontal45"), 
                                                    rm.GetString("Horizontal30"), 
                                                    rm.GetString("Horizontal"), 
                                                    rm.GetString("Horizontal-30"), 
                                                    rm.GetString("Horizontal-45"), 
                                                    rm.GetString("Horizontal-60"),
                                                    rm.GetString("Horizontal-90"),
                                                    rm.GetString("Vertical") });
            this.cmbOwnerTextOrientation.SelectedIndex = 0;

            // Populate the Appearances dropdown
            this.cmbAppearances.Items.AddRange(new string[] { 
                                                    rm.GetString("Column Header Appearance"), 
                                                    rm.GetString("Owner Header Appearance"), 
                                                    rm.GetString("Active Owner Header Appearance"), 
                                                    rm.GetString("Non-Working Hour Appearance"), 
                                                    rm.GetString("Working Hour Appearance"), 
                                                    rm.GetString("Selected DateTime Range Appearance"), 
                                                    rm.GetString("PrimaryInterval's Date Navigation Button Appearance"), 
                                                    rm.GetString("PrimaryInterval's Header Appearance"),
                                                    rm.GetString("PrimartInterval's Hot Tracking Header Appearance") });

            // Set the formatted tooltips
            SetToolTips();
        }
        #endregion InitializeSampleControls

        #region SetDataBindings
        /// <summary>
        /// Set the DataBindings for Appointments and Owners
        /// </summary>
        private void SetDataBindings()
        {
            SetAppointmentBindings();
            SetOwnerBindings();
        }
        #endregion SetDataBindings

        #region SetAppointmentBindings
        /// <summary>
        /// Set the Data Bindings for Appointments
        /// </summary>
        private void SetAppointmentBindings()
        {
            // DataSource & DataMember
            this.ultraCalendarInfo1.DataBindingsForAppointments.SetDataBinding(WinScheduleDataCreationUtils.ScheduleData, "Appointments");

            // Basic Appointment properties
            this.ultraCalendarInfo1.DataBindingsForAppointments.SubjectMember = "Subject";
            this.ultraCalendarInfo1.DataBindingsForAppointments.DescriptionMember = "Description";

            this.ultraCalendarInfo1.DataBindingsForAppointments.StartDateTimeMember = "StartDateTime";
            this.ultraCalendarInfo1.DataBindingsForAppointments.EndDateTimeMember = "EndDateTime";
            this.ultraCalendarInfo1.DataBindingsForAppointments.AllDayEventMember = "AllDayEvent";

            this.ultraCalendarInfo1.DataBindingsForAppointments.ReminderEnabledMember = "ReminderEnabled";
            this.ultraCalendarInfo1.DataBindingsForAppointments.ReminderIntervalMember = "ReminderInterval";
            this.ultraCalendarInfo1.DataBindingsForAppointments.ReminderUnitsMember = "ReminderUnits";

            //Properties when using owners
            this.ultraCalendarInfo1.DataBindingsForAppointments.OwnerKeyMember = "OwnerKey";

            //Properties when using recurring appointments. 
            this.ultraCalendarInfo1.DataBindingsForAppointments.RecurrenceIdMember = "RecurrenceId";
            this.ultraCalendarInfo1.DataBindingsForAppointments.RecurrenceMember = "Recurrence";
            this.ultraCalendarInfo1.DataBindingsForAppointments.OriginalStartDateTimeMember = "OriginalStartDateTime";

            // All other properties
            this.ultraCalendarInfo1.DataBindingsForAppointments.AllPropertiesMember = "AllProperties";

            //DataKey is an extra field that can be used to link the appointment to some external data. 
            this.ultraCalendarInfo1.DataBindingsForAppointments.DataKeyMember = "DataKey";
        }

        #endregion SetAppointmentBindings

        #region SetOwnerBindings
        /// <summary>
        /// Set the Data Bindings for Owners
        /// </summary>
        private void SetOwnerBindings()
        {
            // DataSource & DataMember
            this.ultraCalendarInfo1.DataBindingsForOwners.SetDataBinding(WinScheduleDataCreationUtils.ScheduleData, "Owners");

            // Basic Owner properties
            this.ultraCalendarInfo1.DataBindingsForOwners.KeyMember = "Key";
            this.ultraCalendarInfo1.DataBindingsForOwners.NameMember = "Name";
            this.ultraCalendarInfo1.DataBindingsForOwners.EmailAddressMember = "EmailAddress";
            this.ultraCalendarInfo1.DataBindingsForOwners.VisibleMember = "Visible";

            // All other properties
            this.ultraCalendarInfo1.DataBindingsForOwners.AllPropertiesMember = "AllProperties";

        }
        #endregion SetOwnerBindings

        #endregion Initialization

        #region Event Handlers

        private void Form1_Load(object sender, EventArgs e)
        {
            // Set the databindings for the Appointments and Owners
            SetDataBindings();

            // Add 10 Holidays to the UltraCalendarInfo
            WinScheduleDataCreationUtils.CreateHolidays(this.ultraCalendarInfo1, 10);

            // Initialize the sample controls.
            InitializeSampleControls();
        }

        private void lvAppointmentsPerOwner_SelectedIndexChanged(object sender, EventArgs e)
        {
            ListView listView = (ListView)sender;
            if (listView.SelectedItems.Count == 0)
                return;

            // Bring the Appointment selected in the listview into view in the UltraTimelineView
            this.ultraTimelineView1.EnsureAppointmentVisible((Appointment)listView.SelectedItems[0].Tag);
        }

        private void ultraTimelineView1_ActiveOwnerChanged(object sender, ActiveOwnerChangedEventArgs e)
        {
            // Keep the Owners combo synchronized with the ActiveOwner
            this.cmbOwners.SelectedItem = e.NewActiveOwner.Key;
        }

        private void cmbOwners_SelectedValueChanged(object sender, EventArgs e)
        {
            // Keep the Appointments listview up-to-date with the newly selected owner.
            PopulateAppointmentListView();
        }

        private void ultraCalendarInfo1_AfterAppointmentRemoved(object sender, EventArgs e)
        {
            // Keep the Appointments listview up-to-date when an Appointment is removed.
            PopulateAppointmentListView();
        }

        private void ultraCalendarInfo1_AfterAppointmentAdded(object sender, AppointmentEventArgs e)
        {
            // Keep the Appointments listview up-to-date when an Appointment is added.
            PopulateAppointmentListView();
        }

        private void ultraTimelineView1_AppointmentsDragDrop(object sender, AppointmentsDragDropEventArgs e)
        {
            // Keep the Appointments listview up-to-date when an Appointment is moved.
            PopulateAppointmentListView();
        }

        private void cmbPrimaryInterval_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox combo = (ComboBox)sender;
            int numberOfMinutes = 15;
            switch (combo.SelectedIndex)
            {
                case 0: //"5 Minutes"
                    numberOfMinutes = 5;
                    break;
                case 1: //"15 Minutes"
                    numberOfMinutes = 15;
                    break;
                case 2: //"30 Minutes"
                    numberOfMinutes = 30;
                    break;
                case 3: //"1 Hour"
                    numberOfMinutes = 60;
                    break;
                case 4: //"2 Hours"
                    numberOfMinutes = 120;
                    break;
                case 5: //"4 Hours"
                    numberOfMinutes = 240;
                    break;
                case 6: //"6 Hours"
                    numberOfMinutes = 360;
                    break;
                case 7: //"1 Day"
                    numberOfMinutes = 1440;
                    break;
            }

            // Set the Interval on the PrimaryInterval to the selected number of minutes.
            this.ultraTimelineView1.PrimaryInterval.Interval = numberOfMinutes;
        }

        private void cbDayIntervals_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            if (checkBox.Checked)
            {
                // Add a DateInterval to the AdditionalIntervals collection.
                DateTimeInterval dayInterval = new Infragistics.Win.UltraWinSchedule.DateInterval();
                dayInterval.DateNavigationButtonVisibility = TimelineViewDateNavigationButtonVisibility.ShowOnFirstAndLastHeader;
                this.ultraTimelineView1.AdditionalIntervals.Add(dayInterval);
            }
            else
            {
                // Remove the DateInterval from the AdditionalIntervals collection.
                this.ultraTimelineView1.AdditionalIntervals.Clear();
            }
        }

        private void clbAppointmentDisplayItems_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            // Modify the appropriate AppointmentDisplaySettings property based on the checked/unchecked item.
            switch (e.Index)
            {
                case 0: //Subject
                    this.ultraTimelineView1.AppointmentDisplaySettings.DisplaySubject = ConvertBooleanToDefaultableBoolean(e.NewValue == CheckState.Checked); ;
                    break;
                case 1: //Location
                    this.ultraTimelineView1.AppointmentDisplaySettings.DisplayLocation = ConvertBooleanToDefaultableBoolean(e.NewValue == CheckState.Checked); ;
                    break;
                case 2: //Time
                    this.ultraTimelineView1.AppointmentDisplaySettings.TimeVisibility = e.NewValue == CheckState.Checked ? TimelineViewAppointmentTimeVisibility.BothOnRight : TimelineViewAppointmentTimeVisibility.None;
                    break;
                case 3: //Reminder Indicator
                    this.ultraTimelineView1.AppointmentDisplaySettings.ReminderIndicatorVisibility = e.NewValue == CheckState.Checked ? TimelineViewReminderIndicatorVisibility.Left : TimelineViewReminderIndicatorVisibility.None;
                    break;
                case 4: //Date Time Continuation Indicator
                    this.ultraTimelineView1.AppointmentDisplaySettings.DateTimeContinuationIndicatorVisibility = e.NewValue == CheckState.Checked ? TimelineViewDateTimeContinuationIndicatorVisibility.Both : TimelineViewDateTimeContinuationIndicatorVisibility.None;
                    break;
                case 5: //BarColor Indicator
                    this.ultraTimelineView1.AppointmentDisplaySettings.BarColorIndicatorVisibility = e.NewValue == CheckState.Checked ? TimelineViewBarColorIndicatorVisibility.ShowOnTop : TimelineViewBarColorIndicatorVisibility.Hide;
                    break;
            }
        }

        private void cmbAppointmentDragMode_SelectedValueChanged(object sender, EventArgs e)
        {
            ComboBox combo = (ComboBox)sender;
            // Set the AppointmentDragMode property
            this.ultraTimelineView1.AppointmentDragMode = (TimelineViewAppointmentDragMode)combo.SelectedItem;
        }

        private void cmbAppointmentTooltips_SelectedValueChanged(object sender, EventArgs e)
        {
            ComboBox combo = (ComboBox)sender;
            // Set the AppointmentTooltipVisibility property
            this.ultraTimelineView1.AppointmentToolTipVisibility = (TimelineViewActivityToolTipVisibility)combo.SelectedItem;
        }

        private void cmbAppointmentVisibility_SelectedValueChanged(object sender, EventArgs e)
        {
            ComboBox combo = (ComboBox)sender;
            // Set the AppointmentVisibility property
            this.ultraTimelineView1.AppointmentVisibility = (TimelineViewAppointmentVisibility)combo.SelectedItem;
        }

        private void cbAutoAppointmentDialog_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            // Set the AutoAppointmentDialog property
            this.ultraTimelineView1.AutoAppointmentDialog = checkBox.Checked;
        }

        private void cbAutoAppointmentCreate_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            // Set the AutoAppointmentCreate property
            this.ultraTimelineView1.AutoAppointmentCreate = checkBox.Checked;
        }

        private void cbAppointmentResizing_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            // Set the AllowAppointmentResize property
            this.ultraTimelineView1.AllowAppointmentResize = checkBox.Checked;
        }

        private void cbAlignAppointmentWithInterval_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            // Set the AlignActivityWithPrimaryInterval property
            this.ultraTimelineView1.AlignActivityWithPrimaryInterval = ConvertBooleanToDefaultableBoolean(checkBox.Checked);
        }

        private void cmbColumnSizing_SelectedValueChanged(object sender, EventArgs e)
        {
            ComboBox combo = (ComboBox)sender;
            // Set the ColumnSizing property
            this.ultraTimelineView1.ColumnSizing = (TimelineViewColumnSizing)combo.SelectedItem;
        }

        private void cmbColumnAutoSizing_SelectedValueChanged(object sender, EventArgs e)
        {
            ComboBox combo = (ComboBox)sender;
            // Set the ColumnAutoSizing property
            this.ultraTimelineView1.ColumnAutoSizing = (TimelineViewColumnAutoSizing)combo.SelectedItem;
        }

        private void nmColumnWidth_ValueChanged(object sender, EventArgs e)
        {
            NumericUpDown numericUpDown = (NumericUpDown)sender;
            // Set the ColumnWidth property
            this.ultraTimelineView1.ColumnWidth = Convert.ToInt32(numericUpDown.Value);
        }

        private void ultraTimelineView1_PropertyChanged(object sender, Infragistics.Win.PropertyChangedEventArgs e)
        {
            switch ((TimelineViewPropertyIds)e.ChangeInfo.PropId)
            {
                case TimelineViewPropertyIds.ColumnWidth:
                    // Keep the ColumnWidth editor synchronized with the UltraTimelineView
                    if (this.ultraTimelineView1.ColumnWidthResolved > this.nmColumnWidth.Maximum)
                    {
                        this.ultraTimelineView1.ColumnWidth = Convert.ToInt32(this.nmColumnWidth.Value);
                        MessageBox.Show(rm.GetString("ColumnWidthExceedsMaxError"));
                    }
                    else
                        this.nmColumnWidth.Value = this.ultraTimelineView1.ColumnWidthResolved;
                    break;
                case TimelineViewPropertyIds.MaximumOwnersInView:
                    // Keep the OwnersInView editor synchronized with the UltraTimelineView
                    if (this.ultraTimelineView1.MaximumOwnersInView > this.nmOwnersInView.Maximum)
                    {
                        this.ultraTimelineView1.MaximumOwnersInView = Convert.ToInt32(this.nmOwnersInView.Value);
                        MessageBox.Show(rm.GetString("NumberOfOwnersExceedMaxError"));
                    }
                    else
                        this.nmOwnersInView.Value = this.ultraTimelineView1.OwnersInView;
                    break;
            }
        }

        private void cbColumnScrollbarVisibility_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            // Set the ColumnScrollBarVisibility property
            this.ultraTimelineView1.ColumnScrollBarVisibility = checkBox.Checked ? TimelineViewColumnScrollBarVisibility.Show : TimelineViewColumnScrollBarVisibility.Hide;
        }

        private void nmOwnersInView_ValueChanged(object sender, EventArgs e)
        {
            NumericUpDown numericUpDown = (NumericUpDown)sender;
            // Set the MaximumOwnersInView property
            this.ultraTimelineView1.MaximumOwnersInView = Convert.ToInt32(this.nmOwnersInView.Value);

        }

        private void cmbOwnerGroupingStyle_SelectedValueChanged(object sender, EventArgs e)
        {
            ComboBox combo = (ComboBox)sender;
            // Set the OwnerGroupingStyle property
            this.ultraTimelineView1.OwnerGroupingStyle = (TimelineViewOwnerGroupingStyle)combo.SelectedItem;
        }

        private void cmbHolidayVisibility_SelectedValueChanged(object sender, EventArgs e)
        {
            ComboBox combo = (ComboBox)sender;
            // Set the HolidayVisibility property
            this.ultraTimelineView1.HolidayVisibility = (TimelineViewHolidayVisibility)combo.SelectedItem;
        }

        private void lbHolidays_SelectedValueChanged(object sender, EventArgs e)
        {
            ListBox listBox = (ListBox)sender;
            DateTime holidayDate;

            // Check to see if a date was selected in the Holidays listbox.
            if (listBox.SelectedItem != null &&
                DateTime.TryParse((string)listBox.SelectedItem, out holidayDate))
            {
                // Scroll the selected date into view
                this.ultraTimelineView1.EnsureDateTimeVisible(holidayDate);
            }
        }

        private void cmbHolidayTooltipVisibility_SelectedValueChanged(object sender, EventArgs e)
        {
            ComboBox combo = (ComboBox)sender;
            // Set the HolidayToolTipVisibility property
            this.ultraTimelineView1.HolidayToolTipVisibility = (TimelineViewActivityToolTipVisibility)combo.SelectedItem;
        }

        private void cbOwnerScrollbar_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            // Set the OwnerScrollbarVisibility property
            this.ultraTimelineView1.OwnerScrollBarVisibility = checkBox.Checked ? TimelineViewOwnerScrollBarVisibility.ShowOnlyWhenNeeded : TimelineViewOwnerScrollBarVisibility.Hide;
        }

        private void cmbOwnerSizing_SelectedValueChanged(object sender, EventArgs e)
        {
            ComboBox combo = (ComboBox)sender;
            // Set the OwnerSizing property
            this.ultraTimelineView1.OwnerSizing = (TimelineViewOwnerSizing)combo.SelectedItem;
        }

        private void cmbViewStyle_SelectedValueChanged(object sender, EventArgs e)
        {
            ComboBox combo = (ComboBox)sender;
            // Set the ViewStyle property
            this.ultraCalendarLook1.ViewStyle = (ViewStyle)combo.SelectedItem;
        }

        private void cmbAppearances_SelectedIndexChanged(object sender, EventArgs e)
        {
            string description;
            ComboBox combo = (ComboBox)sender;

            // Retrieve the Appearance object based on the selected index, and set it to the PropertyGrid
            this.pgAppearance.SelectedObject = RetrieveAppearanceBasedOnIndex(combo.SelectedIndex, out description);

            // Update the Appearance Description label
            this.lblAppearanceDescription.Text = description;
        }

        private void cmbOwnerTextOrientation_SelectedValueChanged(object sender, EventArgs e)
        {
            TextOrientationInfo textOrientationInfo = null;
            // Create a new TextOrientationInfo object based on the selected index.
            switch (((ComboBox)sender).SelectedIndex)
            {
                case 0: // "Horizontal : 90 degrees"
                    textOrientationInfo = new TextOrientationInfo(90, TextFlowDirection.Horizontal);
                    break;
                case 1: // "Horizontal : 60 degrees"
                    textOrientationInfo = new TextOrientationInfo(60, TextFlowDirection.Horizontal);
                    break;
                case 2: // "Horizontal : 45 degrees"
                    textOrientationInfo = new TextOrientationInfo(45, TextFlowDirection.Horizontal);
                    break;
                case 3: // "Horizontal : 30 degrees"
                    textOrientationInfo = new TextOrientationInfo(30, TextFlowDirection.Horizontal);
                    break;
                case 4: // "Horizontal"
                    textOrientationInfo = new TextOrientationInfo(0, TextFlowDirection.Horizontal);
                    break;
                case 5: // "Horizontal : -30 degrees"
                    textOrientationInfo = new TextOrientationInfo(-30, TextFlowDirection.Horizontal);
                    break;
                case 6: // "Horizontal : -45 degrees"
                    textOrientationInfo = new TextOrientationInfo(-45, TextFlowDirection.Horizontal);
                    break;
                case 7: // "Horizontal : -60 degrees"
                    textOrientationInfo = new TextOrientationInfo(-60, TextFlowDirection.Horizontal);
                    break;
                case 8: // "Horizontal : -90 degrees"
                    textOrientationInfo = new TextOrientationInfo(-90, TextFlowDirection.Horizontal);
                    break;
                case 9: // "Vertical"
                    textOrientationInfo = new TextOrientationInfo(0, TextFlowDirection.Vertical);
                    break;
            }

            // Assign the new TextOrientationInfo to the OwnerHeaderTextOrientation property
            this.ultraTimelineView1.OwnerHeaderTextOrientation = textOrientationInfo;
        }

        private void ultraCalendarInfo1_AfterSelectedAppointmentsChange(object sender, EventArgs e)
        {
            UltraCalendarInfo ultraCalendarInfo = (UltraCalendarInfo)sender;
            if (ultraCalendarInfo.SelectedAppointments.Count > 0)
            {
                // Attempt to find the selected appointment in the Appointment listbox
                Appointment appointment = ultraCalendarInfo.SelectedAppointments[0];
                foreach (ListViewItem item in this.lvAppointmentsPerOwner.Items)
                    if (ReferenceEquals(item.Tag, appointment))
                    {
                        this.lvAppointmentsPerOwner.SelectedItems.Clear();
                        item.Selected = true;
                        break;
                    }
            }
        }

        private void ultraCalendarInfo1_AppointmentDataInitialized(object sender, AppointmentDataInitializedEventArgs e)
        {
            // For this sample, we don't want to add a location to every bound Appointment
            if (rnd.Next(0, 2) % 2 == 1)
                return;

            // Added a location to the databound Appointment.
            e.Appointment.Location =  rm.GetString("Conference Room") + Char.ConvertFromUtf32(rnd.Next(65, 70));
        }

        #endregion Event Handlers

        #region Private Methods

        private void PopulateAppointmentListView()
        {
            this.lvAppointmentsPerOwner.Items.Clear();

            if (this.cmbOwners.SelectedIndex < 0)
                return;

            string selectedOwnerKey = this.cmbOwners.SelectedValue as string;

            // Ensure the Owner selected in the dropdown is visible in the UltraTimelineView
            int ownerIndex = this.ultraCalendarInfo1.Owners.IndexOf(selectedOwnerKey);
            if (ownerIndex >= 0)
                this.ultraTimelineView1.EnsureOwnerVisible(this.ultraCalendarInfo1.Owners[ownerIndex], false, true);

            // Populate the Appointment ListView
            foreach (Appointment appointment in this.ultraCalendarInfo1.Appointments)
            {
                if (appointment.OwnerKey == selectedOwnerKey)
                {
                    ListViewItem item = new ListViewItem();
                    item.Tag = appointment;
                    item.Text = appointment.Subject;
                    item.ToolTipText = string.Format("{0} - {1}", appointment.Start, appointment.End);

                    this.lvAppointmentsPerOwner.Items.Add(item);
                }
            }
        }

        private AppearanceBase RetrieveAppearanceBasedOnIndex(int index, out string description)
        {
            // Retrieve the appropriate Appearance object (and description) based on the provided index.
            switch (index)
            {
                case 0 : // "Active Owner Header Appearance"
                    description = rm.GetString("DescriptionActiveOwnerHeaderAppearance"); 
                    return this.ultraTimelineView1.ActiveOwnerHeaderAppearance;
                case 1: // "Column Header Appearance"
                    description = rm.GetString("DescriptionColumnHeaderAppearance"); 
                    return this.ultraTimelineView1.ColumnHeaderAppearance;
                case 2: // "Non-Working Hour Appearance"
                    description = rm.GetString("DescriptionNonWorkingHourAppearance"); 
                    return this.ultraTimelineView1.NonWorkingHourAppearance;
                case 3: // "Owner Header Appearance"
                    description = rm.GetString("DescriptionOwnerHeaderAppearance"); 
                    return this.ultraTimelineView1.OwnerHeaderAppearance;
                case 4: // "Selected DateTime Range Appearance"
                    description = rm.GetString("DescriptionSelectedDateTimeRangeAppearance"); 
                    return this.ultraTimelineView1.SelectedDateTimeRangeAppearance;
                case 5: // "Working Hour Appearance"
                    description = rm.GetString("DescriptionWorkingHourAppearance"); 
                    return this.ultraTimelineView1.WorkingHourAppearance;
                case 6: // "PrimaryInterval's Date Navigation Button Appearance"
                    description = rm.GetString("DescriptionPrimaryIntervalDateNavigationButtonAppearance"); 
                    return this.ultraTimelineView1.PrimaryInterval.DateNavigationButtonAppearance;
                case 7: // "PrimaryInterval's Header Appearance"
                    description = rm.GetString("DescriptionPrimaryIntervalHeaderAppearance"); 
                    return this.ultraTimelineView1.PrimaryInterval.HeaderAppearance;
                case 8: // "PrimartInterval's Hot Tracking Header Appearance"
                    description = rm.GetString("DescriptionPrimaryIntervalHotTrackingHeaderAppearance"); 
                    return this.ultraTimelineView1.PrimaryInterval.HeaderHotTrackingAppearance;
                default:
                    description = rm.GetString("DescriptionDefault"); 
                    return null;
            }
        }

        private void SetToolTips()
        {
            FormattedToolTipUtils.LocalizeStrings(rm.GetString("Class"), rm.GetString("Property"));

            UltraToolTipInfo toolTipInfo;

            // ViewStyle and Appearances tab
            this.ultraToolTipManager1.SetUltraToolTip(this.cbColumnScrollbarVisibility, FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "ColumnScrollBarVisibility", rm.GetString("Tooltip_cbColumnScrollbarVisibility")));
            this.ultraToolTipManager1.SetUltraToolTip(this.cbOwnerScrollbar, FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "OwnerScrollBarVisibility", rm.GetString("Tooltip_cbOwnerScrollbar")));
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraCalendarInfo", "ViewStyle", rm.GetString("Tooltip_cmbViewStyle"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbViewStyle, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label15, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbAppearances, FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Information, string.Empty, string.Empty, rm.GetString("Tooltip_cmbAppearances")));

            // Appointment/Activity Related Tab
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "AppointmentDragMode", rm.GetString("Tooltip_cmbAppointmentDragMode"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbAppointmentDragMode, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label2, toolTipInfo);
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "AppointmentTooltipVisibility", rm.GetString("Tooltip_cmbAppointmentTooltips"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbAppointmentTooltips, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label3, toolTipInfo);
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "AppointmentVisibility", rm.GetString("Tooltip_cmbAppointmentVisibility"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbAppointmentVisibility, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label4, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.cbAutoAppointmentDialog, FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "AutoAppointmentDialog", rm.GetString("Tooltip_cbAutoAppointmentDialog")));
            this.ultraToolTipManager1.SetUltraToolTip(this.cbAutoAppointmentCreate, FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "AutoAppointmentCreate", rm.GetString("Tooltip_cbAutoAppointmentCreate")));
            this.ultraToolTipManager1.SetUltraToolTip(this.cbAppointmentResizing, FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "AllowAppointmentResize", rm.GetString("Tooltip_cbAppointmentResizing")));
            this.ultraToolTipManager1.SetUltraToolTip(this.cbAlignAppointmentWithInterval, FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "AlignActivityWithPrimaryInterval", rm.GetString("Tooltip_cbAlignAppointmentWithInterval")));
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "AppointmentDisplaySettings", rm.GetString("Tooltip_clbAppointmentDisplayItems"));
            this.ultraToolTipManager1.SetUltraToolTip(this.clbAppointmentDisplayItems, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.groupBox2, toolTipInfo);

            // Owners Related tab
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "MaximumOwnersInView", rm.GetString("Tooltip_nmOwnersInView"));
            this.ultraToolTipManager1.SetUltraToolTip(this.nmOwnersInView, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label8, toolTipInfo);
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "OwnerGroupingStyle", rm.GetString("Tooltip_cmbOwnerGroupingStyle"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbOwnerGroupingStyle, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label9, toolTipInfo);
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "OwnerSizing", rm.GetString("Tooltip_cmbOwnerSizing"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbOwnerSizing, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label12, toolTipInfo);
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "OwnerHeaderTextOrientation", rm.GetString("Tooltip_cmbOwnerTextOrientation"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbOwnerTextOrientation, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label18, toolTipInfo);
        
            // Interval/Columns Related tab
            this.ultraToolTipManager1.SetUltraToolTip(this.cbDayIntervals, FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "AdditionalIntervals", rm.GetString("Tooltip_cbDayIntervals")));
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "PrimaryInterval", rm.GetString("Tooltip_cmbPrimaryInterval"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbPrimaryInterval, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label1, toolTipInfo);
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "ColumnSizing", rm.GetString("Tooltip_cmbColumnSizing"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbColumnSizing, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label5, toolTipInfo);
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "ColumnAutoSizing", rm.GetString("Tooltip_cmbColumnAutoSizing"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbColumnAutoSizing, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label6, toolTipInfo);
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "ColumnWidth", rm.GetString("Tooltip_nmColumnWidth"));
            this.ultraToolTipManager1.SetUltraToolTip(this.nmColumnWidth, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label7, toolTipInfo);

            // Holidays tab
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "HolidayVisibility", rm.GetString("Tooltip_cmbHolidayVisibility"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbHolidayVisibility, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label10, toolTipInfo);
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Property, "UltraTimeLineView", "HolidayToolTipVisibility", rm.GetString("Tooltip_cmbHolidayTooltipVisibility"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbHolidayTooltipVisibility, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.label11, toolTipInfo);
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Information, string.Empty, string.Empty, rm.GetString("Tooltip_lbHolidays"));
            this.ultraToolTipManager1.SetUltraToolTip(this.lbHolidays, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.groupBox5, toolTipInfo);

            // Other controls
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Information, string.Empty, string.Empty, rm.GetString("Tooltip_cmbOwners"));
            this.ultraToolTipManager1.SetUltraToolTip(this.cmbOwners, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.groupBox3, toolTipInfo);
            toolTipInfo = FormattedToolTipUtils.GenerateFormattedTooltip(FormattedToolTipUtils.ResourceImageType.Information, string.Empty, string.Empty, rm.GetString("Tooltip_lvAppointmentsPerOwner"));
            this.ultraToolTipManager1.SetUltraToolTip(this.lvAppointmentsPerOwner, toolTipInfo);
            this.ultraToolTipManager1.SetUltraToolTip(this.groupBox4, toolTipInfo);
        }

        private DefaultableBoolean ConvertBooleanToDefaultableBoolean(bool value)
        {
            return value ? DefaultableBoolean.True : DefaultableBoolean.False;
        }

        #endregion Private Methods
    }
}